# encoding: utf-8
# kanakana.py - hiragana <-> katakana conversion module
# 
# Copyright (c) 2005 M.Shibata <mshibata@emptypage.jp>
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to 
# deal in the Software without restriction, including without limitation the 
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or 
# sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in 
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
# IN THE SOFTWARE.
# 


_usage = """usage: kanakana.py -m mode [-e encoding] filename
    mode:   hiragana2katakana or h2k
            katakana2hiragana or k2h
            zenkaku2hankaku or fullwidth2halfwidth or z2h or f2h
            hankaku2zenkaku or halfwidth2fullwidth or h2z or h2f
"""


hiragana_to_katakana = {
    u'ぁ': u'ァ',
    u'あ': u'ア',
    u'ぃ': u'ィ',
    u'い': u'イ',
    u'ぅ': u'ゥ',
    u'う': u'ウ',
    u'ぇ': u'ェ',
    u'え': u'エ',
    u'ぉ': u'ォ',
    u'お': u'オ',
    u'か': u'カ',
    u'が': u'ガ',
    u'き': u'キ',
    u'ぎ': u'ギ',
    u'く': u'ク',
    u'ぐ': u'グ',
    u'け': u'ケ',
    u'げ': u'ゲ',
    u'こ': u'コ',
    u'ご': u'ゴ',
    u'さ': u'サ',
    u'ざ': u'ザ',
    u'し': u'シ',
    u'じ': u'ジ',
    u'す': u'ス',
    u'ず': u'ズ',
    u'せ': u'セ',
    u'ぜ': u'ゼ',
    u'そ': u'ソ',
    u'ぞ': u'ゾ',
    u'た': u'タ',
    u'だ': u'ダ',
    u'ち': u'チ',
    u'ぢ': u'ヂ',
    u'っ': u'ッ',
    u'つ': u'ツ',
    u'づ': u'ヅ',
    u'て': u'テ',
    u'で': u'デ',
    u'と': u'ト',
    u'ど': u'ド',
    u'な': u'ナ',
    u'に': u'ニ',
    u'ぬ': u'ヌ',
    u'ね': u'ネ',
    u'の': u'ノ',
    u'は': u'ハ',
    u'ば': u'バ',
    u'ぱ': u'パ',
    u'ひ': u'ヒ',
    u'び': u'ビ',
    u'ぴ': u'ピ',
    u'ふ': u'フ',
    u'ぶ': u'ブ',
    u'ぷ': u'プ',
    u'へ': u'ヘ',
    u'べ': u'ベ',
    u'ぺ': u'ペ',
    u'ほ': u'ホ',
    u'ぼ': u'ボ',
    u'ぽ': u'ポ',
    u'ま': u'マ',
    u'み': u'ミ',
    u'む': u'ム',
    u'め': u'メ',
    u'も': u'モ',
    u'ゃ': u'ャ',
    u'や': u'ヤ',
    u'ゅ': u'ュ',
    u'ゆ': u'ユ',
    u'ょ': u'ョ',
    u'よ': u'ヨ',
    u'ら': u'ラ',
    u'り': u'リ',
    u'る': u'ル',
    u'れ': u'レ',
    u'ろ': u'ロ',
    u'ゎ': u'ヮ',
    u'わ': u'ワ',
    u'ゐ': u'ヰ',
    u'ゑ': u'ヱ',
    u'を': u'ヲ',
    u'ん': u'ン',
    u'う゛': u'ヴ',
    u'ゝ': u'ヽ',
    u'ゞ': u'ヾ',
}

katakana_to_hiragana = {
    u'ァ': u'ぁ',
    u'ア': u'あ',
    u'ィ': u'ぃ',
    u'イ': u'い',
    u'ゥ': u'ぅ',
    u'ウ': u'う',
    u'ェ': u'ぇ',
    u'エ': u'え',
    u'ォ': u'ぉ',
    u'オ': u'お',
    u'カ': u'か',
    u'ガ': u'が',
    u'キ': u'き',
    u'ギ': u'ぎ',
    u'ク': u'く',
    u'グ': u'ぐ',
    u'ケ': u'け',
    u'ゲ': u'げ',
    u'コ': u'こ',
    u'ゴ': u'ご',
    u'サ': u'さ',
    u'ザ': u'ざ',
    u'シ': u'し',
    u'ジ': u'じ',
    u'ス': u'す',
    u'ズ': u'ず',
    u'セ': u'せ',
    u'ゼ': u'ぜ',
    u'ソ': u'そ',
    u'ゾ': u'ぞ',
    u'タ': u'た',
    u'ダ': u'だ',
    u'チ': u'ち',
    u'ヂ': u'ぢ',
    u'ッ': u'っ',
    u'ツ': u'つ',
    u'ヅ': u'づ',
    u'テ': u'て',
    u'デ': u'で',
    u'ト': u'と',
    u'ド': u'ど',
    u'ナ': u'な',
    u'ニ': u'に',
    u'ヌ': u'ぬ',
    u'ネ': u'ね',
    u'ノ': u'の',
    u'ハ': u'は',
    u'バ': u'ば',
    u'パ': u'ぱ',
    u'ヒ': u'ひ',
    u'ビ': u'び',
    u'ピ': u'ぴ',
    u'フ': u'ふ',
    u'ブ': u'ぶ',
    u'プ': u'ぷ',
    u'ヘ': u'へ',
    u'ベ': u'べ',
    u'ペ': u'ぺ',
    u'ホ': u'ほ',
    u'ボ': u'ぼ',
    u'ポ': u'ぽ',
    u'マ': u'ま',
    u'ミ': u'み',
    u'ム': u'む',
    u'メ': u'め',
    u'モ': u'も',
    u'ャ': u'ゃ',
    u'ヤ': u'や',
    u'ュ': u'ゅ',
    u'ユ': u'ゆ',
    u'ョ': u'ょ',
    u'ヨ': u'よ',
    u'ラ': u'ら',
    u'リ': u'り',
    u'ル': u'る',
    u'レ': u'れ',
    u'ロ': u'ろ',
    u'ヮ': u'ゎ',
    u'ワ': u'わ',
    u'ヰ': u'ゐ',
    u'ヱ': u'ゑ',
    u'ヲ': u'を',
    u'ン': u'ん',
    u'ヴ': u'う゛',
    u'ヽ': u'ゝ',
    u'ヾ': u'ゞ',
}

fullwidth_to_halfwidth_kana = {
    u'ー': u'ｰ',
    u'゛': u'ﾞ',
    u'゜': u'ﾟ',
    u'ァ': u'ｧ',
    u'ア': u'ｱ',
    u'ィ': u'ｨ',
    u'イ': u'ｲ',
    u'ゥ': u'ｩ',
    u'ウ': u'ｳ',
    u'ェ': u'ｪ',
    u'エ': u'ｴ',
    u'ォ': u'ｫ',
    u'オ': u'ｵ',
    u'カ': u'ｶ',
    u'ガ': u'ｶﾞ',
    u'キ': u'ｷ',
    u'ギ': u'ｷﾞ',
    u'ク': u'ｸ',
    u'グ': u'ｸﾞ',
    u'ケ': u'ｹ',
    u'ゲ': u'ｹﾞ',
    u'コ': u'ｺ',
    u'ゴ': u'ｺﾞ',
    u'サ': u'ｻ',
    u'ザ': u'ｻﾞ',
    u'シ': u'ｼ',
    u'ジ': u'ｼﾞ',
    u'ス': u'ｽ',
    u'ズ': u'ｽﾞ',
    u'セ': u'ｾ',
    u'ゼ': u'ｾﾞ',
    u'ソ': u'ｿ',
    u'ゾ': u'ｿﾞ',
    u'タ': u'ﾀ',
    u'ダ': u'ﾀﾞ',
    u'チ': u'ﾁ',
    u'ヂ': u'ﾁﾞ',
    u'ッ': u'ｯ',
    u'ツ': u'ﾂ',
    u'ヅ': u'ﾂﾞ',
    u'テ': u'ﾃ',
    u'デ': u'ﾃﾞ',
    u'ト': u'ﾄ',
    u'ド': u'ﾄﾞ',
    u'ナ': u'ﾅ',
    u'ニ': u'ﾆ',
    u'ヌ': u'ﾇ',
    u'ネ': u'ﾈ',
    u'ノ': u'ﾉ',
    u'ハ': u'ﾊ',
    u'バ': u'ﾊﾞ',
    u'パ': u'ﾊﾟ',
    u'ヒ': u'ﾋ',
    u'ビ': u'ﾋﾞ',
    u'ピ': u'ﾋﾟ',
    u'フ': u'ﾌ',
    u'ブ': u'ﾌﾞ',
    u'プ': u'ﾌﾟ',
    u'ヘ': u'ﾍ',
    u'ベ': u'ﾍﾞ',
    u'ペ': u'ﾍﾟ',
    u'ホ': u'ﾎ',
    u'ボ': u'ﾎﾞ',
    u'ポ': u'ﾎﾟ',
    u'マ': u'ﾏ',
    u'ミ': u'ﾐ',
    u'ム': u'ﾑ',
    u'メ': u'ﾒ',
    u'モ': u'ﾓ',
    u'ャ': u'ｬ',
    u'ヤ': u'ﾔ',
    u'ュ': u'ｭ',
    u'ユ': u'ﾕ',
    u'ョ': u'ｮ',
    u'ヨ': u'ﾖ',
    u'ラ': u'ﾗ',
    u'リ': u'ﾘ',
    u'ル': u'ﾙ',
    u'レ': u'ﾚ',
    u'ロ': u'ﾛ',
    u'ヮ': u'ﾜ',
    u'ワ': u'ﾜ',
    u'ヰ': u'ｲ',
    u'ヱ': u'ｴ',
    u'ヲ': u'ｦ',
    u'ン': u'ﾝ',
    u'ヴ': u'ｳﾞ',
    u'ヵ': u'ｶ',
    u'ヶ': u'ｹ',
}

halfwidth_to_fullwidth_kana = {
    u'ｰ': u'ー',
    u'ﾞ': u'゛',
    u'ﾟ': u'゜',
    u'ｧ': u'ァ',
    u'ｱ': u'ア',
    u'ｨ': u'ィ',
    u'ｲ': u'イ',
    u'ｩ': u'ゥ',
    u'ｳ': u'ウ',
    u'ｪ': u'ェ',
    u'ｴ': u'エ',
    u'ｫ': u'ォ',
    u'ｵ': u'オ',
    u'ｶ': u'カ',
    u'ｶﾞ': u'ガ',
    u'ｷ': u'キ',
    u'ｷﾞ': u'ギ',
    u'ｸ': u'ク',
    u'ｸﾞ': u'グ',
    u'ｹ': u'ケ',
    u'ｹﾞ': u'ゲ',
    u'ｺ': u'コ',
    u'ｺﾞ': u'ゴ',
    u'ｻ': u'サ',
    u'ｻﾞ': u'ザ',
    u'ｼ': u'シ',
    u'ｼﾞ': u'ジ',
    u'ｽ': u'ス',
    u'ｽﾞ': u'ズ',
    u'ｾ': u'セ',
    u'ｾﾞ': u'ゼ',
    u'ｿ': u'ソ',
    u'ｿﾞ': u'ゾ',
    u'ﾀ': u'タ',
    u'ﾀﾞ': u'ダ',
    u'ﾁ': u'チ',
    u'ﾁﾞ': u'ヂ',
    u'ｯ': u'ッ',
    u'ﾂ': u'ツ',
    u'ﾂﾞ': u'ヅ',
    u'ﾃ': u'テ',
    u'ﾃﾞ': u'デ',
    u'ﾄ': u'ト',
    u'ﾄﾞ': u'ド',
    u'ﾅ': u'ナ',
    u'ﾆ': u'ニ',
    u'ﾇ': u'ヌ',
    u'ﾈ': u'ネ',
    u'ﾉ': u'ノ',
    u'ﾊ': u'ハ',
    u'ﾊﾞ': u'バ',
    u'ﾊﾟ': u'パ',
    u'ﾋ': u'ヒ',
    u'ﾋﾞ': u'ビ',
    u'ﾋﾟ': u'ピ',
    u'ﾌ': u'フ',
    u'ﾌﾞ': u'ブ',
    u'ﾌﾟ': u'プ',
    u'ﾍ': u'ヘ',
    u'ﾍﾞ': u'ベ',
    u'ﾍﾟ': u'ペ',
    u'ﾎ': u'ホ',
    u'ﾎﾞ': u'ボ',
    u'ﾎﾟ': u'ポ',
    u'ﾏ': u'マ',
    u'ﾐ': u'ミ',
    u'ﾑ': u'ム',
    u'ﾒ': u'メ',
    u'ﾓ': u'モ',
    u'ｬ': u'ャ',
    u'ﾔ': u'ヤ',
    u'ｭ': u'ュ',
    u'ﾕ': u'ユ',
    u'ｮ': u'ョ',
    u'ﾖ': u'ヨ',
    u'ﾗ': u'ラ',
    u'ﾘ': u'リ',
    u'ﾙ': u'ル',
    u'ﾚ': u'レ',
    u'ﾛ': u'ロ',
    u'ﾜ': u'ワ',
    u'ｴ': u'ヱ',
    u'ｦ': u'ヲ',
    u'ﾝ': u'ン',
    u'ｳﾞ': u'ヴ',
}

hiragana_uppercase = {
    u'ぁ': u'あ',
    u'ぃ': u'い',
    u'ぅ': u'う',
    u'ぇ': u'え',
    u'ぉ': u'お',
    u'っ': u'つ',
    u'ゃ': u'や',
    u'ゅ': u'ゆ',
    u'ょ': u'よ',
    u'ゎ': u'わ',
}

katakana_uppercase = {
    u'ァ': u'ア',
    u'ィ': u'イ',
    u'ゥ': u'ウ',
    u'ェ': u'エ',
    u'ォ': u'オ',
    u'ッ': u'ツ',
    u'ャ': u'ヤ',
    u'ュ': u'ユ',
    u'ョ': u'ヨ',
    u'ヮ': u'ワ',
}

halfwidth_katakana_uppercase = {
    u'ｧ': u'ｱ',
    u'ｨ': u'ｲ',
    u'ｩ': u'ｳ',
    u'ｪ': u'ｴ',
    u'ｫ': u'ｵ',
    u'ｯ': u'ﾂ',
    u'ｬ': u'ﾔ',
    u'ｭ': u'ﾕ',
    u'ｮ': u'ﾖ',
}


fullwidth_to_halfwidth_punc = {
    u'　': u' ',
    u'。': u'｡',
    u'「': u'｢',
    u'」': u'｣',
    u'、': u'､',
    u'・': u'･',
    u'，': u',',
    u'．': u'.',
    u'：': u':',
    u'；': u';',
    u'？': u'?',
    u'！': u'!',
    u'＾': u'^',
    u'＿': u'_',
    u'‐': u'-',
    u'／': u'/',
    u'～': u'~',
    u'（': u'(',
    u'）': u')',
    u'［': u'[',
    u'］': u']',
    u'｛': u'{',
    u'｝': u'}',
    u'＋': u'+',
    u'－': u'-',
    u'＝': u'=',
    u'＜': u'<',
    u'＞': u'>',
    u'￥': u'\u00a5',
    u'＄': u'$',
    u'％': u'%',
    u'＃': u'#',
    u'＆': u'&',
    u'＊': u'*',
    u'＠': u'@',
}

halfwidth_to_fullwidth_punc = {
    u' ': u'　',
    u'｡': u'。',
    u'｢': u'「',
    u'｣': u'」',
    u'､': u'、',
    u'･': u'・',
    u',': u'，',
    u'.': u'．',
    u':': u'：',
    u';': u'；',
    u'?': u'？',
    u'!': u'！',
    u'^': u'＾',
    u'_': u'＿',
    u'-': u'‐',
    u'/': u'／',
    u'~': u'～',
    u'(': u'（',
    u')': u'）',
    u'[': u'［',
    u']': u'］',
    u'{': u'｛',
    u'}': u'｝',
    u'+': u'＋',
    u'-': u'－',
    u'=': u'＝',
    u'<': u'＜',
    u'>': u'＞',
    u'\u00a5': u'￥',
    u'$': u'＄',
    u'%': u'％',
    u'#': u'＃',
    u'&': u'＆',
    u'*': u'＊',
    u'@': u'＠',
}

fullwidth_to_halfwidth_alphabet = {
    u'Ａ': u'A',
    u'Ｂ': u'B',
    u'Ｃ': u'C',
    u'Ｄ': u'D',
    u'Ｅ': u'E',
    u'Ｆ': u'F',
    u'Ｇ': u'G',
    u'Ｈ': u'H',
    u'Ｉ': u'I',
    u'Ｊ': u'J',
    u'Ｋ': u'K',
    u'Ｌ': u'L',
    u'Ｍ': u'M',
    u'Ｎ': u'N',
    u'Ｏ': u'O',
    u'Ｐ': u'P',
    u'Ｑ': u'Q',
    u'Ｒ': u'R',
    u'Ｓ': u'S',
    u'Ｔ': u'T',
    u'Ｕ': u'U',
    u'Ｖ': u'V',
    u'Ｗ': u'W',
    u'Ｘ': u'X',
    u'Ｙ': u'Y',
    u'Ｚ': u'Z',
    u'ａ': u'a',
    u'ｂ': u'b',
    u'ｃ': u'c',
    u'ｄ': u'd',
    u'ｅ': u'e',
    u'ｆ': u'f',
    u'ｇ': u'g',
    u'ｈ': u'h',
    u'ｉ': u'i',
    u'ｊ': u'j',
    u'ｋ': u'k',
    u'ｌ': u'l',
    u'ｍ': u'm',
    u'ｎ': u'n',
    u'ｏ': u'o',
    u'ｐ': u'p',
    u'ｑ': u'q',
    u'ｒ': u'r',
    u'ｓ': u's',
    u'ｔ': u't',
    u'ｕ': u'u',
    u'ｖ': u'v',
    u'ｗ': u'w',
    u'ｘ': u'x',
    u'ｙ': u'y',
    u'ｚ': u'z',
}

halfwidth_to_fullwidth_alphabet = {
    u'A': u'Ａ',
    u'B': u'Ｂ',
    u'C': u'Ｃ',
    u'D': u'Ｄ',
    u'E': u'Ｅ',
    u'F': u'Ｆ',
    u'G': u'Ｇ',
    u'H': u'Ｈ',
    u'I': u'Ｉ',
    u'J': u'Ｊ',
    u'K': u'Ｋ',
    u'L': u'Ｌ',
    u'M': u'Ｍ',
    u'N': u'Ｎ',
    u'O': u'Ｏ',
    u'P': u'Ｐ',
    u'Q': u'Ｑ',
    u'R': u'Ｒ',
    u'S': u'Ｓ',
    u'T': u'Ｔ',
    u'U': u'Ｕ',
    u'V': u'Ｖ',
    u'W': u'Ｗ',
    u'X': u'Ｘ',
    u'Y': u'Ｙ',
    u'Z': u'Ｚ',
    u'a': u'ａ',
    u'b': u'ｂ',
    u'c': u'ｃ',
    u'd': u'ｄ',
    u'e': u'ｅ',
    u'f': u'ｆ',
    u'g': u'ｇ',
    u'h': u'ｈ',
    u'i': u'ｉ',
    u'j': u'ｊ',
    u'k': u'ｋ',
    u'l': u'ｌ',
    u'm': u'ｍ',
    u'n': u'ｎ',
    u'o': u'ｏ',
    u'p': u'ｐ',
    u'q': u'ｑ',
    u'r': u'ｒ',
    u's': u'ｓ',
    u't': u'ｔ',
    u'u': u'ｕ',
    u'v': u'ｖ',
    u'w': u'ｗ',
    u'x': u'ｘ',
    u'y': u'ｙ',
    u'z': u'ｚ',
}

fullwidth_to_halfwidth_num = {
    u'１': u'1',
    u'２': u'2',
    u'３': u'3',
    u'４': u'4',
    u'５': u'5',
    u'６': u'6',
    u'７': u'7',
    u'８': u'8',
    u'９': u'9',
    u'０': u'0' 
}

halfwidth_to_fullwidth_num = {
    u'1': u'１',
    u'2': u'２',
    u'3': u'３',
    u'4': u'４',
    u'5': u'５',
    u'6': u'６',
    u'7': u'７',
    u'8': u'８',
    u'9': u'９',
    u'0': u'０'
}


def init_table(table):
    def cmp_by_len(a, b):
        return len(b) - len(a)
    
    table['__keys__'] = table.keys()
    table['__keys__'].sort(cmp_by_len)


for table in (hiragana_to_katakana,
              katakana_to_hiragana,
              hiragana_uppercase,
              katakana_uppercase,
              halfwidth_katakana_uppercase,
              fullwidth_to_halfwidth_kana,
              halfwidth_to_fullwidth_kana,
              fullwidth_to_halfwidth_punc,
              halfwidth_to_fullwidth_punc,
              fullwidth_to_halfwidth_alphabet,
              halfwidth_to_fullwidth_alphabet,
              fullwidth_to_halfwidth_num,
              halfwidth_to_fullwidth_num):
    init_table(table)


def apply_table(src, *tables):
    for table in tables:
        dst = u''
        i = 0
        while i < len(src):
            tmp = src[i]
            for key in table['__keys__']:
                if src[i:].startswith(key):
                    tmp = table[key]
                    i += len(key) - 1
                    break
            dst += tmp
            i += 1
        src = dst
    return src


def hiragana2katakana(src):
    return apply_table(src, hiragana_to_katakana)


def katakana2hiragana(src):
    return apply_table(src, katakana_to_hiragana)


def uppercase(src):
    return apply_table(src, hiragana_uppercase,
                            katakana_uppercase,
                            halfwidth_katakana_uppercase)


def fullwidth2halfwidth(src):
    return apply_table(src, fullwidth_to_halfwidth_kana,
                            fullwidth_to_halfwidth_punc,
                            fullwidth_to_halfwidth_alphabet,
                            fullwidth_to_halfwidth_num)

zenkaku2hankaku = fullwidth2halfwidth


def halfwidth2fullwidth(src):
    return apply_table(src, halfwidth_to_fullwidth_kana,
                            halfwidth_to_fullwidth_punc,
                            halfwidth_to_fullwidth_alphabet,
                            halfwidth_to_fullwidth_num)

hankaku2zenkaku = halfwidth2fullwidth


if __name__ == '__main__':
    import sys, getopt
    
    try:
        opts, args = getopt.getopt(sys.argv[1:], "m:e:")
    except getopt.GetoptError:
        print _usage
        sys.exit(2)
    
    encoding = sys.stdin.encoding
    funcs = {'hiragana2katakana': hiragana2katakana,
             'h2k': hiragana2katakana,
             'katakana2hiragana': katakana2hiragana,
             'k2h': hiragana2katakana,
             'fullwidth2halfwidth': fullwidth2halfwidth,
             'zenkaku2hankaku': fullwidth2halfwidth,
             'f2h': fullwidth2halfwidth,
             'z2h': fullwidth2halfwidth,
             'halfwidth2fullwidth': halfwidth2fullwidth,
             'hankaku2zenkaku': halfwidth2fullwidth,
             'h2f': halfwidth2fullwidth,
             'h2z': halfwidth2fullwidth}
    conv = None
    for k, v in opts:
        if k == "-e":
            encoding = v
        if k == "-m":
            try:
                conv = funcs[v]
            except KeyError:
                print _usage
                sys.exit(2)
    
    if conv and args:
        for filename in args:
            for line in file(args.pop()):
                print conv(unicode(line, encoding)).encode(encoding),
    else:
        print _usage

